// Plots a spectral age map - If the age range is greater than 15, it is best to use smooth maps

int errormap(float *ageerrors, int **regionarray, float **flux, int xaxismax, int yaxismax, char *settarget, int border, float zoom, int titles, int labels, int axismarks, int extwincontrol, int smoothmaps, int contours, int contourlevels, int contourcolour, int contourlinestyle, int firstcontour, int contourlogs, char *output, double inject, double fieldstrength, int paramlabels, int model, float ra, float dec, float cellsize, int scaletype, float bmaj, float bmin, float bpa, int beamposx, int beamposy, int plotbeam, float wedgemultiplier, int xshift, int yshift, int errorpm) {

#include "smoothmaps.h"

  int mapscale(int scaletype, int istart, int iend, int jstart, int jend, int border, float cellsize, int axismarks, int labels, float ra, float dec, char *maptitle, int xshift, int yshift);

  int drawbeam(float xpos, float ypos, float bmaj, float bmin, float bpa);

  // Declare the local variables
  int i, j, istart, jstart, iend, jend;
  int intenscale = 0;
  float zoomedx, zoomedy, adj_maxrange;
  char specmaptitle[100];
  char str_inject[64];
  char str_field[64];

  // Delcare pgpplot prototypes
  int cpgopen(const char *device);
  void cpgclos(void);
  void cpgpt1(float xpt, float ypt, int symbol);
  void cpgsci(int ci);
  void cpgimag(const float *a, int idim, int jdim, int i1, int i2, int j1, int j2, float a1, float a2, const float *tr);
  void cpgctab(const float *l, const float *r, const float *g, const float *b, int nc, float contra, float bright);
  void cpgwedg(const char *side, float disp, float width, float fg, float bg, const char *label);
  void cpgscr(int ci, float cr, float cg, float cb);
  void cpgcont(const float *a, int idim, int jdim, int i1, int i2, int j1, int j2, const float *c, int nc, const float *tr);
  void cpgsls(int ls);
  void cpgmtxt(const char *side, float disp, float coord, float fjust, const char *text);


  if (extwincontrol != 1) { 
    cpgopen(output);
  }

  // Swap to background and forground colours to something sensible
  cpgscr(0, 1, 1, 1);
  cpgscr(1, 0, 0, 0);

  // Set plot title
  if (titles == 0) { sprintf(specmaptitle, " "); }
  else if (errorpm == 0) { sprintf(specmaptitle, "Positive Error Map of %s", settarget); }
  else { sprintf(specmaptitle, "Negative Error Map of %s", settarget); }



  cpgsci(1);

 // Calculate the zoomed start and end points
  zoomedx = xaxismax / zoom;
  zoomedy = yaxismax / zoom;

  istart = xshift + border + ( (xaxismax - (int)zoomedx) / 2);
  jstart = yshift + border + ( (yaxismax - (int)zoomedy) / 2);

  iend = xshift + xaxismax - ( ( (xaxismax - (int)zoomedx) / 2) + border);
  jend = yshift + yaxismax - ( ( (yaxismax - (int)zoomedy) / 2) + border);


  // Set plot title
  if (titles == 0) { sprintf(specmaptitle, " "); }
  else { sprintf(specmaptitle, "Error Map of %s", settarget); }


  mapscale(scaletype, istart, iend, jstart, jend, border, cellsize, axismarks, labels, ra, dec, specmaptitle, xshift, yshift);


  if (smoothmaps == 1)  {

    float *tmp_smootharray;
    float maxrange = -1e23;
    float minrange = 1e23;


    tmp_smootharray = (float *)calloc(xaxismax*yaxismax, sizeof(float));


    for (i=0; i<xaxismax; i++) {
      for (j=0; j<yaxismax; j++) {
	
	if (regionarray[i][j] < 0) { // Set void regions to a high value to create a white background
	  tmp_smootharray[i*xaxismax+j] = 1e24;
	}

	else {

	  tmp_smootharray[i*xaxismax+j] = ageerrors[regionarray[i][j]];

	  if (tmp_smootharray[i*xaxismax+j] < minrange) {
	    minrange = tmp_smootharray[i*xaxismax+j];
	  }

	  if (tmp_smootharray[i*xaxismax+j] > maxrange) {
	    maxrange = tmp_smootharray[i*xaxismax+j];
	  }
	}

      }
    }

    adj_maxrange = maxrange * wedgemultiplier;

    if (adj_maxrange < 0.01) { // this prevents a 0 range, especially when using one region
      printf("\n*** Warning: Range is too small, setting maxrange to a value of 0.01\n\n"); 
      adj_maxrange = 0.01;
    }

    cpgctab(rl,rr,rg,rb,11,contra,bright);
    cpgwedg("RI", 1.5, 5.0, minrange, adj_maxrange, "Error (Myr)");
    cpgimag(tmp_smootharray,xaxismax,yaxismax,istart,iend,jstart,jend,minrange,adj_maxrange,trans);

    free(tmp_smootharray);
  }

  else {

    // Do the old style mapping
    for (i=istart; i<iend; i++){
      for (j=jstart; j<jend; j++){
	if (regionarray[i][j] <= 0) {
	  intenscale = 0;
	}
	else {
	  switch ((int)ageerrors[regionarray[i][j]]) {
	  case 0:
	    intenscale = 1;
	    break;
	  case 1:
	    intenscale = 2;
	    break;
	  case 2:
	    intenscale = 3;
	    break; 
	  case 3:
	    intenscale = 4;
	    break;
	  case 4:
	    intenscale = 5;
	    break;
	  case 5:
	    intenscale = 6;
	    break;
	  case 6:
	    intenscale = 7;
	    break;
	  case 7:
	    intenscale = 8;
	    break;
	  case 8:
	    intenscale = 9;
	    break;
	  case 9:
	    intenscale = 10;
	    break;
	  case 10:
	    intenscale = 11;
	    break;
	  case 11:
	    intenscale = 12;
	    break;
	  case 12:
	    intenscale = 13;
	    break;
	  case 13:
	    intenscale = 14;
	    break;
	  case 14:
	    intenscale = 15;
	    break;
	  case '?':
	    intenscale = 0;
	    break;
	  }
	}

	cpgsci(intenscale);
	cpgpt1(j,i,1);

      }
    }
  }


  if (contours == 1) {

    float *tmp_contourarray;
    float minrange = 1e32;
    float maxrange = -1e32;
    float currentcont;


    cpgsci(contourcolour);
    cpgsls(contourlinestyle);

    tmp_contourarray = (float *)calloc(xaxismax*yaxismax, sizeof(float));

    for (i=0; i<xaxismax; i++) {
      for (j=0; j<yaxismax; j++) {

       	if (regionarray[i][j] > 0) { // Stop non-included regions from creating a false minimum

	  tmp_contourarray[i*xaxismax+j] = flux[i][j];

	  if (tmp_contourarray[i*xaxismax+j] < minrange) {
	    minrange = tmp_contourarray[i*xaxismax+j];
	  }

	  if (tmp_contourarray[i*xaxismax+j] > maxrange) {
	    maxrange = tmp_contourarray[i*xaxismax+j];
	  }
	}

	else {
	  tmp_contourarray[i*xaxismax+j] = 0.0;
	}

      }
    }


    for (i=firstcontour; i<=contourlevels; i++) {

      if (contourlogs == 1) {
	currentcont = pow(10, log10(minrange) + (i*(log10(maxrange)-log10(minrange))/contourlevels) );
      }
      else {
	currentcont = minrange + i*(maxrange-minrange)/contourlevels;
      }

      cpgcont(tmp_contourarray, xaxismax, yaxismax, istart, iend, jstart, jend, &currentcont, -1, trans);
    }

    free(tmp_contourarray);
  }

  if (paramlabels == 1) {

    if (model == 1) {
      cpgmtxt("B", -3.5, 0.65, 0.0, "JP Model");
    }
    if (model == 2) {
      cpgmtxt("B", -3.5, 0.65, 0.0, "KP Model");
    }
    if (model == 3) {
      cpgmtxt("B", -3.5, 0.65, 0.0, "Tribble Model (JP)");
    }
    else if (model == 4) {
      cpgmtxt("B", -3.5, 0.65, 0.0, "Tribble Model (KP)");
    }

    sprintf(str_inject, "Injection index: %.2f", inject);
    sprintf(str_field, "B-Field: %.2e T", fieldstrength);

    cpgmtxt("B", -2.5 , 0.65, 0.0, str_inject);
    cpgmtxt("B", -1.5 , 0.65, 0.0, str_field);

  }

  // Plot the beam if the option is set to on
  if (plotbeam == 1) {
    int beamposi = istart-border+(bmaj/cellsize)+beamposx;
    int beamposj = jstart-border+(bmaj/cellsize)+beamposy;

    // This must be after the main plotting to create an overlay
    drawbeam(beamposi, beamposj, bmaj/cellsize, bmin/cellsize, bpa);
  }

  if (extwincontrol != 1) {
    cpgclos();
  }

  return 0;

}
